@@ -22,6 +22,10 @@
 #include "lyst.h"
 #include "sdrxn.h"
 
+#ifndef SDR_SEMKEY
+#define SDR_SEMKEY	(0xeee0)
+#endif
+
 static PsmPartition	_sdrwm(sm_WmParms *parms);
 
 #ifndef SDR_TRACE
@@ -56,7 +60,33 @@ static char	*_SdrSchName()
 	return "sdrsch";
 }
 
-/*	*	*	SDR working memory area.	*	*	*/
+/*	*	*	SDR global system variables	*	*	*/
+
+static sm_SemId	_sdrlock(int delete)
+{
+	/*	This is the mutex for the SDR system as a whole; it's
+	 *	global to all SDR operations on any single computer.	*/
+
+	static sm_SemId	lock = SM_SEM_NONE;
+
+	if (delete)
+	{
+		if (lock != SM_SEM_NONE)
+		{
+			sm_SemDelete(lock);
+			lock = SM_SEM_NONE;
+		}
+	}
+	else
+	{
+		if (lock == SM_SEM_NONE)
+		{
+			lock = sm_SemCreate(SDR_SEMKEY, SM_SEM_FIFO);
+		}
+	}
+
+	return lock;
+}
 
 static int	_sdrMemory(int *memmgrIdx)
 {
@@ -85,6 +115,7 @@ static long	_sdrwmSize(long *wmsize)
 static SdrControlHeader	*_sch(SdrControlHeader **schp)
 {
 	static SdrControlHeader	*sch = NULL;
+	sm_SemId		lock;
 	PsmPartition		sdrwm;
 	PsmAddress		elt;
 	SdrState		*sdr;
@@ -107,31 +138,31 @@ static SdrControlHeader	*_sch(SdrControlHeader **schp)
 		}
 
 		sdrwm = _sdrwm(NULL);
-		if (sch->lock != -1)
+		lock = _sdrlock(0);
+		if (lock != SM_SEM_NONE)
 		{
 			/*	Note: wait until sdrs list is no
 			 *	longer in use before ending access
-			 *	to the SDRs and destroying the sch
-			 *	semaphore.				*/
+			 *	to the SDRs.				*/
 
-			sm_SemTake(sch->lock);
+			sm_SemTake(lock);
 			for (elt = sm_list_first(sdrwm, sch->sdrs); elt;
 					elt = sm_list_next(sdrwm, elt))
 			{
 				sdr = (SdrState *)
 					psp(sdrwm, sm_list_data(sdrwm, elt));
 				CHKNULL(sdr);
-				if (sdr->sdrSemaphore != -1)
+				if (sdr->sdrSemaphore != SM_SEM_NONE)
 				{
 					sm_SemTake(sdr->sdrSemaphore);
 					sm_SemEnd(sdr->sdrSemaphore);
 					microsnooze(500000);
 					sm_SemDelete(sdr->sdrSemaphore);
-					sdr->sdrSemaphore = -1;
+					sdr->sdrSemaphore = SM_SEM_NONE;
 				}
 			}
 
-			sm_SemDelete(sch->lock);
+			sm_SemGive(lock);
 		}
 
 		oK(psm_uncatlg(sdrwm, _SdrSchName()));
@@ -153,7 +184,6 @@ static SdrControlHeader	*_sch(SdrControlHeader **schp)
 	{
 		sch = (SdrControlHeader *) psp(sdrwm, controlHeaderAddress);
 		CHKNULL(sch);
-		sm_SemUnwedge(sch->lock, 3);
 		sm_list_unwedge(sdrwm, sch->sdrs, 3);
 		return sch;
 	}
@@ -171,13 +201,6 @@ static SdrControlHeader	*_sch(SdrControlHeader **schp)
 
 	sch = (SdrControlHeader *) psp(sdrwm, controlHeaderAddress);
 	CHKNULL(sch);
-	sch->lock = sm_SemCreate(SM_NO_KEY, SM_SEM_FIFO);
-	if (sch->lock == SM_SEM_NONE)
-	{
-		putErrmsg("Failed to create SDR lock semaphore.", NULL);
-		return NULL;		/*	Can't create semaphore.	*/
-	}
-
 	sch->sdrs = sm_list_create(sdrwm);
 	if (sch->sdrs == 0)
 	{
@@ -404,14 +427,10 @@ void	releaseSdr(SdrState *sdr)
 
 int	Sdr_initialize(long wmSize, char *wmPtr, int wmKey, char *wmName)
 {
+	sm_SemId	lock;
 	sm_WmParms	wmparms;
 	PsmPartition	sdrwm;
 
-	if (_sdrwm(NULL) != NULL)	/*	Already initialized.	*/
-        {
-		return 0;
-	}
-
 	/*	Initialize the shared-memory libraries as necessary.	*/
 
 	if (sm_ipc_init())
@@ -420,13 +439,28 @@ int	Sdr_initialize(long wmSize, char *wmPtr, int wmKey, char *wmName)
 		return -1;	/*	Crash if can't initialize sm.	*/
 	}
 
+	lock = _sdrlock(0);
+	if (lock == SM_SEM_NONE
+	|| sm_SemUnwedge(lock, 3) < 0 || sm_SemTake(lock) < 0)
+	{
+		putErrmsg("Can't initialize SDR system.", NULL);
+		return -1;	/*	Crash if can't lock sdr.	*/
+	}
+
 	/*	Attach to SDR's own shared working memory partition.	*/
+
+	if (_sdrwm(NULL) != NULL)	/*	Already initialized.	*/
+        {
+		sm_SemGive(lock);
+		return 0;
+	}
    
 	wmparms.wmKey = wmKey;
 	wmparms.wmSize = wmSize;
 	wmparms.wmAddress = wmPtr;
 	wmparms.wmName = wmName;
 	sdrwm = _sdrwm(&wmparms);
+	sm_SemGive(lock);
 	if (sdrwm == NULL)
 	{
 		putErrmsg("Can't open SDR working memory.", NULL);
@@ -444,15 +478,23 @@ void	sdr_wm_usage(PsmUsageSummary *usage)
 
 void	sdr_shutdown()		/*	Ends SDR service on machine.	*/
 {
+	sm_SemId	lock;
 	sm_WmParms	wmparms;
 
-	if (_sdrwm(NULL) == NULL)	/*	Nothing to do.		*/
+	lock = _sdrlock(0);
+	if (lock == SM_SEM_NONE || sm_SemTake(lock) < 0)
 	{
-		return;
+		putErrmsg("SDR system not initialized, can't shut down.", NULL);
+		return;	/*	Crash if can't lock sdr.	*/
 	}
 
-	wmparms.wmSize = -1;
-	oK(_sdrwm(&wmparms));
+	if (_sdrwm(NULL) != NULL)
+	{
+		wmparms.wmSize = -1;
+		oK(_sdrwm(&wmparms));
+	}
+
+	_sdrlock(1);		/*	Delete SDR system semaphore.	*/
 }
 
 int	_xniEnd(const char *fileName, int lineNbr, const char *arg, Sdr sdrv)
@@ -862,6 +904,7 @@ static int	createDbFile(SdrState *sdr, char *dbfilename)
 int	sdr_load_profile(char *name, int configFlags, long heapWords,
 		int memKey, char *pathName)
 {
+	sm_SemId		lock = _sdrlock(0);
 	PsmPartition		sdrwm = _sdrwm(NULL);
 	SdrControlHeader	*sch = _sch(NULL);
 	PsmAddress		elt;
@@ -888,7 +931,12 @@ int	sdr_load_profile(char *name, int configFlags, long heapWords,
 		return -1;
 	}
 
-	sm_SemTake(sch->lock);
+	if (lock == SM_SEM_NONE || sm_SemTake(lock) < 0)
+	{
+		putErrmsg("Can't lock SDR control header.", NULL);
+		return -1;
+	}
+
 	for (elt = sm_list_first(sdrwm, sch->sdrs); elt;
 			elt = sm_list_next(sdrwm, elt))
 	{
@@ -896,7 +944,7 @@ int	sdr_load_profile(char *name, int configFlags, long heapWords,
 		CHKERR(sdr);
 		if (strcmp(sdr->name, name) == 0)
 		{
-			sm_SemGive(sch->lock);
+			sm_SemGive(lock);
 			if (sdr->configFlags == configFlags
 			&& sdr->initHeapWords == heapWords
 			&& (sdr->sdrKey == memKey || memKey == SM_NO_KEY)
@@ -916,7 +964,7 @@ int	sdr_load_profile(char *name, int configFlags, long heapWords,
 	newSdrAddress = psm_zalloc(sdrwm, sizeof(SdrState));
 	if (newSdrAddress == 0)
 	{
-		sm_SemGive(sch->lock);
+		sm_SemGive(lock);
 		putErrmsg("Can't allocate memory for SDR state.", NULL);
 		return -1;
 	}
@@ -939,7 +987,7 @@ int	sdr_load_profile(char *name, int configFlags, long heapWords,
 	if (sdr->sdrSemaphore == SM_SEM_NONE)
 	{
 		psm_free(sdrwm, newSdrAddress);
-		sm_SemGive(sch->lock);
+		sm_SemGive(lock);
 		putErrmsg("Can't create transaction semaphore for SDR.", NULL);
 		return -1;
 	}
@@ -964,7 +1012,7 @@ in file and transaction reversibility", sdr->pathName);
 	if (sdr->sdrsElt == 0)
 	{
 		psm_free(sdrwm, newSdrAddress);
-		sm_SemGive(sch->lock);
+		sm_SemGive(lock);
 		putErrmsg("Can't insert SDR state into list of SDRs.", NULL);
 		return -1;
 	}
@@ -980,7 +1028,7 @@ in file and transaction reversibility", sdr->pathName);
 		if (logfile == -1)
 		{
 			psm_free(sdrwm, newSdrAddress);
-			sm_SemGive(sch->lock);
+			sm_SemGive(lock);
 			putSysErrmsg("Can't open log file", logfilename);
 			return -1;
 		}
@@ -990,7 +1038,7 @@ in file and transaction reversibility", sdr->pathName);
 		{
 			close(logfile);
 			psm_free(sdrwm, newSdrAddress);
-			sm_SemGive(sch->lock);
+			sm_SemGive(lock);
 			putErrmsg(_noMemoryMsg(), NULL);
 			return -1;
 		}
@@ -1000,7 +1048,7 @@ in file and transaction reversibility", sdr->pathName);
 			lyst_destroy(logEntries);
 			close(logfile);
 			psm_free(sdrwm, newSdrAddress);
-			sm_SemGive(sch->lock);
+			sm_SemGive(lock);
 			putErrmsg("Can't reload log entries.", NULL);
 			return -1;
 		}
@@ -1019,7 +1067,7 @@ in file and transaction reversibility", sdr->pathName);
 				if (logfile != -1) close(logfile);
 				if (logEntries) lyst_destroy(logEntries);
 				psm_free(sdrwm, newSdrAddress);
-				sm_SemGive(sch->lock);
+				sm_SemGive(lock);
 				putErrmsg("Can't have file-based database",
 						NULL);
 				return -1;
@@ -1034,7 +1082,7 @@ in file and transaction reversibility", sdr->pathName);
 				if (logfile != -1) close(logfile);
 				if (logEntries) lyst_destroy(logEntries);
 				psm_free(sdrwm, newSdrAddress);
-				sm_SemGive(sch->lock);
+				sm_SemGive(lock);
 				putErrmsg("Can't reverse log entries.", NULL);
 				return -1;
 			}
@@ -1051,7 +1099,7 @@ in file and transaction reversibility", sdr->pathName);
 			if (logfile != -1) close(logfile);
 			if (logEntries) lyst_destroy(logEntries);
 			psm_free(sdrwm, newSdrAddress);
-			sm_SemGive(sch->lock);
+			sm_SemGive(lock);
 			putErrmsg("Can't attach to database partition.", NULL);
 			return -1;
 	
@@ -1068,7 +1116,7 @@ in file and transaction reversibility", sdr->pathName);
 					if (logEntries)
 						lyst_destroy(logEntries);
 					psm_free(sdrwm, newSdrAddress);
-					sm_SemGive(sch->lock);
+					sm_SemGive(lock);
 					putErrmsg("Can't load db from file.",
 							NULL);
 					return -1;
@@ -1085,7 +1133,7 @@ in file and transaction reversibility", sdr->pathName);
 				if (logfile != -1) close(logfile);
 				if (logEntries) lyst_destroy(logEntries);
 				psm_free(sdrwm, newSdrAddress);
-				sm_SemGive(sch->lock);
+				sm_SemGive(lock);
 				putErrmsg("Can't reverse log entries.", NULL);
 				return -1;
 			}
@@ -1105,7 +1153,7 @@ in file and transaction reversibility", sdr->pathName);
 					if (logEntries)
 						lyst_destroy(logEntries);
 					psm_free(sdrwm, newSdrAddress);
-					sm_SemGive(sch->lock);
+					sm_SemGive(lock);
 					putErrmsg("Can't load db from file.",
 							NULL);
 					return -1;
@@ -1135,13 +1183,14 @@ in file and transaction reversibility", sdr->pathName);
 		close(dbfile);
 	}
 
-	sm_SemGive(sch->lock);
+	sm_SemGive(lock);
 	return 0;
 }
 
 int	sdr_reload_profile(char *name, int configFlags, long heapWords,
 		int memKey, char *pathName)
 {
+	sm_SemId		lock = _sdrlock(0);
 	PsmPartition		sdrwm = _sdrwm(NULL);
 	SdrControlHeader	*sch = _sch(NULL);
 	PsmAddress		elt;
@@ -1152,7 +1201,12 @@ int	sdr_reload_profile(char *name, int configFlags, long heapWords,
 	CHKERR(sch);
 	CHKERR(name);
 	CHKERR(pathName);
-	sm_SemTake(sch->lock);
+	if (lock == SM_SEM_NONE || sm_SemTake(lock) < 0)
+	{
+		putErrmsg("Can't lock SDR control header.", NULL);
+		return -1;
+	}
+
 	for (elt = sm_list_first(sdrwm, sch->sdrs); elt;
 			elt = sm_list_next(sdrwm, elt))
 	{
@@ -1172,7 +1226,7 @@ int	sdr_reload_profile(char *name, int configFlags, long heapWords,
 		|| (sdr->sdrKey != memKey && memKey != SM_NO_KEY)
 		|| strcmp(sdr->pathName, pathName) != 0)
 		{
-			sm_SemGive(sch->lock);
+			sm_SemGive(lock);
 			putErrmsg("Can't unload SDR: profile conflict.", name);
 			return -1;
 		}
@@ -1189,7 +1243,7 @@ int	sdr_reload_profile(char *name, int configFlags, long heapWords,
 
 	/*	Profile for this SDR is now known to be unloaded.	*/
 
-	sm_SemGive(sch->lock);
+	sm_SemGive(lock);
 	return sdr_load_profile(name, configFlags, heapWords, memKey, pathName);
 }
 
@@ -1200,6 +1254,7 @@ static void	deleteObjectExtent(LystElt elt, void *userData)
 
 Sdr	Sdr_start_using(char *name)
 {
+	sm_SemId		lock = _sdrlock(0);
 	PsmPartition		sdrwm = _sdrwm(NULL);
 	SdrControlHeader	*sch = _sch(NULL);
 	PsmAddress		elt;
@@ -1212,7 +1267,12 @@ Sdr	Sdr_start_using(char *name)
 	CHKNULL(sdrwm);
 	CHKNULL(sch);
 	CHKNULL(name);
-	sm_SemTake(sch->lock);
+	if (lock == SM_SEM_NONE || sm_SemTake(lock) < 0)
+	{
+		putErrmsg("Can't lock SDR control header.", NULL);
+		return NULL;
+	}
+
 	for (elt = sm_list_first(sdrwm, sch->sdrs); elt;
 			elt = sm_list_next(sdrwm, elt))
 	{
@@ -1226,7 +1286,7 @@ Sdr	Sdr_start_using(char *name)
 
 	if (elt == 0)	/*	Reached the end of the list, no match.	*/
 	{
-		sm_SemGive(sch->lock);
+		sm_SemGive(lock);
 		putErrmsg("SDR profile not found.", name);
 		return NULL;
 	}
@@ -1236,7 +1296,7 @@ Sdr	Sdr_start_using(char *name)
 	sdrViewAddress = psm_zalloc(sdrwm, sizeof(SdrView));
 	if (sdrViewAddress == 0)
 	{
-		sm_SemGive(sch->lock);
+		sm_SemGive(lock);
 		putErrmsg(_noMemoryMsg(), NULL);
 		return NULL;
 	}
@@ -1250,7 +1310,7 @@ Sdr	Sdr_start_using(char *name)
 		sdrv->dbfile = iopen(dbfilename, O_RDWR, 0777);
 		if (sdrv->dbfile == -1)
 		{
-			sm_SemGive(sch->lock);
+			sm_SemGive(lock);
 			putSysErrmsg("Can't open database file", dbfilename);
 			return NULL;
 		}
@@ -1266,7 +1326,7 @@ Sdr	Sdr_start_using(char *name)
 		if (sm_ShmAttach(sdr->sdrKey, sdr->sdrSize, &(sdrv->dbsm),
 					&(sdrv->dbsmId)) < 0)
 		{
-			sm_SemGive(sch->lock);
+			sm_SemGive(lock);
 			putErrmsg("Can't attach to database in memory.",
 					itoa(sdr->sdrKey));
 			return NULL;
@@ -1281,7 +1341,7 @@ Sdr	Sdr_start_using(char *name)
 				0777);
 		if (sdrv->logfile == -1)
 		{
-			sm_SemGive(sch->lock);
+			sm_SemGive(lock);
 			putSysErrmsg("Can't open log file", logfilename);
 			return NULL;
 		}
@@ -1289,7 +1349,7 @@ Sdr	Sdr_start_using(char *name)
 		sdrv->logEntries = lyst_create_using(_sdrMemory(NULL));
 		if (sdrv->logEntries == 0)
 		{
-			sm_SemGive(sch->lock);
+			sm_SemGive(lock);
 			putErrmsg(_noMemoryMsg(), NULL);
 			return NULL;
 		}
@@ -1304,7 +1364,7 @@ Sdr	Sdr_start_using(char *name)
 		sdrv->knownObjects = lyst_create_using(_sdrMemory(NULL));
 		if (sdrv->knownObjects == 0)
 		{
-			sm_SemGive(sch->lock);
+			sm_SemGive(lock);
 			putErrmsg(_noMemoryMsg(), NULL);
 			return NULL;
 		}
@@ -1315,7 +1375,7 @@ Sdr	Sdr_start_using(char *name)
 	sdrv->trace = NULL;
 	sdrv->currentSourceFileName = NULL;
 	sdrv->currentSourceFileLine = 0;
-	sm_SemGive(sch->lock);
+	sm_SemGive(lock);
 	return sdrv;
 }
 
@@ -1392,8 +1452,8 @@ void	sdr_abort(Sdr sdrv)
 
 void	sdr_destroy(Sdr sdrv)
 {
+	sm_SemId		lock = _sdrlock(0);
 	PsmPartition		sdrwm = _sdrwm(NULL);
-	SdrControlHeader	*sch = _sch(NULL);
 	SdrState		*sdr;
 	char			dbfilename[PATHLENMAX + 1 + 32 + 1 + 3 + 1];
 	char			logfilename[PATHLENMAX + 1 + 32 + 1 + 6 + 1];
@@ -1417,7 +1477,14 @@ void	sdr_destroy(Sdr sdrv)
 	sdr = sdrv->sdr;
 	sm_SemDelete(sdr->sdrSemaphore);	/*	Interrupt.	*/
 	sdr_stop_using(sdrv);
-	sm_SemTake(sch->lock);
+
+	/*	Destroy global persistent SDR state.			*/
+
+	if (lock == SM_SEM_NONE || sm_SemTake(lock) < 0)
+	{
+		putErrmsg("Can't lock SDR control header.", NULL);
+		return;
+	}
 
 	/*	Destroy file copy of heap if any.			*/
 
@@ -1455,7 +1522,7 @@ void	sdr_destroy(Sdr sdrv)
 
 	oK(sm_list_delete(sdrwm, sdr->sdrsElt, NULL, NULL));
 	psm_free(sdrwm, psa(sdrwm, sdr));
-	sm_SemGive(sch->lock);
+	sm_SemGive(lock);
 }
 
 /*	*	Low-level transaction functions		*	*	*/
